# Generated by Django 3.2.2 on 2021-05-08 10:15

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Platform',
            fields=[
                ('platform_id', models.IntegerField(primary_key=True, serialize=False, verbose_name='平台ID')),
                ('platform_name', models.CharField(max_length=128, verbose_name='平台名称')),
            ],
        ),
    ]



import ast

from django.http import HttpResponse
from rest_framework import response
from rest_framework.permissions import AllowAny
from rest_framework.utils import json
from rest_framework.views import APIView
from django.core.cache import cache
import random
from aliyunsdkcore.client import AcsClient
from aliyunsdkcore.request import CommonRequest
from config.config import Config

# from utils import restful


accesskey_id = Config.getIns().get("xj_user", "ACCESSKEY_ID")
accesskey_secret = Config.getIns().get("xj_user", "ACCESSKEY_SECRET")
sign_name = Config.getIns().get("xj_user", "SIGN_NAME")
template_code = Config.getIns().get("xj_user", "TEMPLATE_CODE")


class UserShortMessage(APIView):

    def post(self, request):
        if not accesskey_id or not accesskey_secret or not sign_name or not template_code:
            res = {
                'err': 4999,
                'msg': "短信参数未配置"
            }
            return response.Response(res)

        # http://localhost:8000/duanxin/duanxin/sms_send/?phone=18434288349
        # 1 获取手机号
        phone = request.POST.get('phone')
        # 2 生成6位验证码
        code = self.get_code(6, False)
        # print(code)
        # 3 缓存到Redis
        cache.set(phone, code, 300)  # 60s有效期
        # print('判断缓存中是否有:', cache.has_key(phone))
        # print('获取Redis验证码:', cache.get(phone))
        # 4 发短信
        result = self.send_sms(phone, code)
        dictionary = ast.literal_eval(result)
        if dictionary['Code'] == 'OK':
            res = {
                'err': 0,
                'msg': 'OK'
            }
        else:
            res = {
                'err': 4999,
                'msg': dictionary['Message']
            }

        return response.Response(res)

    def send_sms(self, phone, code):
        client = AcsClient(accesskey_id, accesskey_secret)
        code = "{'code':%s}" % (code)
        request = CommonRequest()
        request.set_accept_format('json')
        request.set_domain('dysmsapi.aliyuncs.com')  # url
        request.set_method('POST')
        request.set_protocol_type('https')  # https | http
        request.set_version('2017-05-25')
        request.set_action_name('SendSms')
        request.add_query_param('RegionId', 'cn-hangzhou')
        request.add_query_param('PhoneNumbers', phone)  # 待发送手机号
        request.add_query_param('SignName', sign_name)  # 短信签名
        request.add_query_param('TemplateCode', template_code)  # 短信模板code
        request.add_query_param('TemplateParam', code)
        response = client.do_action_with_exception(request)
        # python2: print(response)
        return str(response, encoding='utf-8')

    # 数字表示生成几位, True表示生成带有字母的 False不带字母的
    def get_code(self, n=6, alpha=False):
        s = ''  # 创建字符串变量,存储生成的验证码
        for i in range(n):  # 通过for循环控制验证码位数
            num = random.randint(0, 9)  # 生成随机数字0-9
            if alpha:  # 需要字母验证码,不用传参,如果不需要字母的,关键字alpha=False
                upper_alpha = chr(random.randint(65, 90))
                lower_alpha = chr(random.randint(97, 122))
                num = random.choice([num, upper_alpha, lower_alpha])
            s = s + str(num)
        return s



# Generated by Django 3.2.2 on 2021-05-09 15:00

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('xj_user', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='BaseInfo',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('platform_uid', models.BigIntegerField(db_index=True, verbose_name='平台用户ID')),
                ('user_name', models.CharField(blank=True, db_index=True, max_length=128, null=True, verbose_name='用户名')),
                ('full_name', models.CharField(blank=True, max_length=128, null=True, verbose_name='姓名')),
                ('phone', models.CharField(blank=True, db_index=True, max_length=128, null=True, verbose_name='手机')),
                ('email', models.EmailField(blank=True, db_index=True, max_length=254, null=True, verbose_name='邮箱')),
                ('wechat', models.CharField(blank=True, db_index=True, max_length=128, null=True, verbose_name='微信号')),
                ('user_info', models.JSONField(blank=True, null=True, verbose_name='用户信息')),
                ('platform', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, to='xj_user.platform')),
            ],
            options={
                'verbose_name': '用户基本信息表',
                'db_table': 'user_base_info',
                'unique_together': {('platform', 'user_name'), ('platform', 'platform_uid')},
            },
        ),
    ]



# encoding: utf-8
"""
@project: djangoModel->user_list
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 用户信息列表
@created_time: 2022/7/25 9:42
"""
from rest_framework.views import APIView

from ..services.user_service import UserService
from ..utils.custom_response import util_response
from ..utils.model_handle import parse_data


class UserListAPIView(APIView):
    # 在有的框架中会出现传入3个参数的问题，原因不明，可能和两边使用的版本不同有关
    def get(self, request, *args):
        token = request.META.get('HTTP_AUTHORIZATION', None)
        token_serv, error_text = UserService.check_token(token)
        if error_text:
            return util_response(err=6045, msg=error_text)
        only_field_dict = {
            "page": "page",
            "size": "size",
            "user_id": "id",
            "email": "email",
            "full_name": "full_name__full_name",
            "user_name": "user_name__contains",
            "nickname": "nickname__contains",
        }
        params = parse_data(request, only_field=only_field_dict)
        data, err_txt = UserService.user_list(params)
        if not error_text:
            return util_response(data=data)
        return util_response(err=47767, msg=error_text)



from django.contrib import admin

# 引入用户平台
from .models import *
from config.config import Config

config = Config()

# Register your models here.
admin.site.site_title = config.get('app', 'app_name')  # 登录页面的标题
admin.site.site_header = 'MSA一体化管理系统'  # 登录框的标题
admin.site.index_title = 'MSA一体化管理后台'  # 首页的标题


class BaseInfoAdmin(admin.ModelAdmin):
    # fields = ('user_name', 'full_name', 'platform', 'platform_uid', 'phone', 'email', 'wechat', 'user_info')
    fields = ('user_name', 'full_name', 'nickname', 'phone', 'email', 'wechat_openid', 'user_info', 'user_group', 'permission', 'register_time')
    list_display = ('id', 'user_name', 'full_name', 'nickname', 'phone', 'email', 'wechat_openid', 'user_info', 'user_group', 'permission', 'register_time')
    search_fields = ('user_name', 'full_name', 'nickname', 'phone', 'user_group__group')
    list_filter = []


class AuthAdmin(admin.ModelAdmin):
    fields = ('user', 'password', 'token', 'ticket',)
    list_display = ('user', 'password', 'token', 'create_time', 'update_time',)
    search_fields = ('user', 'create_time')
    list_filter = ['user']

    # def platform(self, obj):
    #     return obj.platform


class DetailInfoAdmin(admin.ModelAdmin):
    fields = (
        'user', 'real_name', 'sex', 'birth', 'tags', 'signature', 'avatar', 'cover', 'language', 'region_code',
        'more',
        'field_1', 'field_2', 'field_3', 'field_4', 'field_5', 'field_6', 'field_7', 'field_8', 'field_9',
        'field_10', 'field_11', 'field_12', 'field_13', 'field_14', 'field_15'
    )
    list_display = ('user', 'real_name', 'sex', 'birth', 'tags', 'avatar', 'cover', 'language', 'region_code')


class ExtendFiledAdmin(admin.ModelAdmin):
    fields = ('field', 'field_index', 'description')
    list_display = ('field', 'field_index', 'description')


class AccessLogAdmin(admin.ModelAdmin):
    fields = ('user', 'ip', 'create_time', 'client_info', 'more',)
    list_display = ('user', 'ip', 'create_time', 'client_info',)


class HistoryAdmin(admin.ModelAdmin):
    fields = ('user', 'field', 'old_value', 'new_value', 'create_time',)
    list_display = ('user', 'field', 'old_value', 'new_value', 'create_time',)


class RestrictRegionAdmin(admin.ModelAdmin):
    fields = ('user', 'region_code',)
    list_display = ('user', 'region_code',)


class PlatformAdmin(admin.ModelAdmin):
    fields = ('platform_id', 'platform_name')
    list_display = ('platform_id', 'platform_name')
    search_fields = ('platform_id', 'platform_name')


class PlatformsToUsersAdmin(admin.ModelAdmin):
    fields = ('platform', 'platform_user_id',)
    list_display = ('platform', 'platform_user_id',)


class PermissionAdmin(admin.ModelAdmin):
    fields = ('permission_id', 'permission_name',)
    list_display = ('permission_id', 'permission_name',)


class PermissionValueAdmin(admin.ModelAdmin):
    fields = ('permission', 'value', 'is_system', 'is_ban',)
    list_display = ('permission', 'value', 'is_system', 'is_ban',)


class GroupAdmin(admin.ModelAdmin):
    fields = ('group', 'parent_group', 'description')
    list_display = ('id', 'group', 'parent_group', 'description')


admin.site.register(BaseInfo, BaseInfoAdmin)
admin.site.register(Auth, AuthAdmin)
admin.site.register(DetailInfo, DetailInfoAdmin)
admin.site.register(AccessLog, AccessLogAdmin)
admin.site.register(History, HistoryAdmin)
admin.site.register(RestrictRegion, RestrictRegionAdmin)
admin.site.register(Platform, PlatformAdmin)
admin.site.register(PlatformsToUsers, PlatformsToUsersAdmin)
admin.site.register(Permission, PermissionAdmin)
admin.site.register(PermissionValue, PermissionValueAdmin)
admin.site.register(Group, GroupAdmin)
admin.site.register(ExtendFiled, ExtendFiledAdmin)



# Generated by Django 3.2.2 on 2021-05-11 04:50

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('xj_user', '0002_baseinfo'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='baseinfo',
            options={'verbose_name_plural': '用户基础信息表'},
        ),
        migrations.AlterModelOptions(
            name='platform',
            options={'verbose_name_plural': '平台基础信息表'},
        ),
        migrations.AlterField(
            model_name='baseinfo',
            name='platform',
            field=models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, to='xj_user.platform', verbose_name='平台ID'),
        ),
        migrations.AlterField(
            model_name='platform',
            name='platform_id',
            field=models.IntegerField(help_text='必填。不自动生成，由运营人员统一设置。', primary_key=True, serialize=False, verbose_name='平台ID'),
        ),
        migrations.AlterField(
            model_name='platform',
            name='platform_name',
            field=models.CharField(help_text='必填。平台名称可以是中文、英文、俄文等。', max_length=128, verbose_name='平台名称'),
        ),
    ]



# Generated by Django 3.2.5 on 2022-02-05 05:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('xj_user', '0005_merge_0004_auto_20220101_1450_0004_auto_20220101_1707'),
        ('xj_user', '0006_auto_20220205_1300'),
    ]

    operations = [
    ]



# encoding: utf-8
"""
@project: djangoModel->user_permission_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 用户权限服务
@created_time: 2022/8/23 9:33
"""
from ..models import BaseInfo


class PermissionService():
    __user_dict = None
    __is_err = False
    __err_message = None

    def __init__(self, user_id):
        self.user_id = user_id

    @property
    def user(self):
        """用户基础信息"""
        if self.__user_dict is None:
            self.__user_dict = BaseInfo.objects.filter(id=self.user_id).first().to_json()
        return self.__user_dict

    @property
    def err_message(self):
        return self.__err_message

    @err_message.setter
    def err_message(self, message):
        self.__is_err = True
        self.__err_message = message

    def is_valid(self):
        """
        判断是否存在异常
        :return Bool 存在异常True,不存在异常False
        """
        return self.__is_err

    def get_err_msg(self):
        """获取错误提示信息"""
        return self.err_message

    def permission_users(self, params=None):
        pass

    def is_allowable(self):
        """判断是否有操作全权限"""
        return True

    def permission_list(self, params=None):
        """获取当前用户的功能权限列表"""
        return None, None



from django.db import models
from django.utils import timezone

# Create your models here.


# 9、User_Permission 权限表 [1-N]
class Permission(models.Model):
    permission_id = models.IntegerField(verbose_name='权限ID', primary_key=True, help_text='必填。不自动生成，由运营人员统一设置。')
    permission_name = models.CharField(verbose_name='权限名称', max_length=255, blank=True, null=True, )

    class Meta:
        db_table = 'user_permission'
        verbose_name_plural = "09. 用户 - 权限类型表"

    def __str__(self):
        # return f"{self.user_name}({self.full_name})"
        return f"{self.permission_name}"


# 10、User_Permission_Value 权限值表 [1-N]
# 权限标识值，一个permission_id可以对应多个value，多值形成一组权限。值为宏名，需要多语言翻译
class PermissionValue(models.Model):
    permission = models.ForeignKey(Permission, verbose_name='权限ID', on_delete=models.DO_NOTHING, )
    value = models.CharField(verbose_name='权限值', max_length=255, blank=True, null=True, )
    is_system = models.BooleanField(verbose_name='是否系统权限', blank=True, null=True, )
    is_ban = models.BooleanField(verbose_name='是否禁用', blank=True, null=True, )

    class Meta:
        db_table = 'user_permission_value'
        verbose_name_plural = "10. 用户 - 权限详细表"

    def __str__(self):
        # return f"{self.user_name}({self.full_name})"
        return f"{self.value}"


# 11、User_Group 分组表
class Group(models.Model):
    id = models.IntegerField(verbose_name='ID', primary_key=True, auto_created=True)
    # xj_user = models.ForeignKey(BaseInfo, verbose_name='用户', on_delete=models.DO_NOTHING, )
    group = models.CharField(verbose_name='用户组', max_length=32, blank=True, null=True, )
    parent_group = models.CharField(verbose_name='父组', max_length=32, blank=True, null=True, )
    description = models.CharField(verbose_name='描述', max_length=32, blank=True, null=True, )

    # permission = models.ForeignKey(Permission, verbose_name='权限ID', on_delete=models.DO_NOTHING, blank=True, null=True, )

    class Meta:
        db_table = 'user_group'
        verbose_name_plural = "11. 用户 - 分组表"

    def __str__(self):
        return f"{self.description}"


# 1、User_Base_Info 基础信息表 [NF1]
class BaseInfo(models.Model):
    # platform_uid = models.BigIntegerField(verbose_name='平台用户ID', db_index=True)
    # platform = models.ForeignKey(Platform, on_delete=models.DO_NOTHING, verbose_name='平台ID',)
    user_name = models.CharField(verbose_name='用户名', max_length=128, blank=True, null=True, db_index=True)
    full_name = models.CharField(verbose_name='姓名', max_length=128, blank=True, null=True, db_index=True)
    nickname = models.CharField(verbose_name='昵称', max_length=128, blank=True, null=True)
    phone = models.CharField(verbose_name='手机', max_length=128, blank=True, null=True, db_index=True)
    email = models.EmailField(verbose_name='邮箱', blank=True, null=True, db_index=True)
    wechat_openid = models.CharField(verbose_name='微信开放号', max_length=128, blank=True, null=True, db_index=True)
    user_info = models.JSONField(verbose_name='用户信息', blank=True, null=True)
    user_group = models.ForeignKey(Group, verbose_name='用户分组', blank=True, null=True, on_delete=models.DO_NOTHING)
    permission = models.ForeignKey(Permission, verbose_name='用户权限', blank=True, null=True, on_delete=models.DO_NOTHING)
    register_time = models.DateTimeField(verbose_name='注册时间', default=timezone.now(), )

    class Meta:
        db_table = 'user_base_info'
        # unique_together = [['platform', 'platform_uid'], ['platform', 'user_name']]
        verbose_name_plural = "01. 用户 - 基础信息"

    def __str__(self):
        # return f"{self.user_name}({self.full_name})"
        return f"{self.user_name}"

    def get_group_desc(self):
        return self.user_group.description if self.user_group else ""

    def get_permission_desc(self):
        return self.permission.permission_name if self.permission else ""


# 2、User_Auth 认证表 [1-1]
class Auth(models.Model):
    user = models.ForeignKey(BaseInfo, verbose_name='用户', unique=True, on_delete=models.DO_NOTHING, )
    password = models.CharField(verbose_name='密码', max_length=255, blank=True, null=True, )
    plaintext = models.CharField(verbose_name='PT', max_length=255, blank=True, null=True, )
    salt = models.CharField(verbose_name='盐', max_length=32, blank=True, null=True, )
    algorithm = models.CharField(verbose_name='加密算法', max_length=255, blank=True, null=True, )
    token = models.CharField(verbose_name='临时凭证', max_length=255, blank=True, null=True, )
    ticket = models.CharField(verbose_name='临时票据', max_length=255, blank=True, null=True, )
    create_time = models.DateTimeField(verbose_name='密码创建时间', auto_now_add=True, )
    update_time = models.DateTimeField(verbose_name='凭证更新时间', auto_now=True, blank=True, null=True, )

    class Meta:
        db_table = 'user_auth'
        verbose_name_plural = "02. 用户 - 安全认证"

    def __str__(self):
        # return f"{self.user_name}({self.full_name})"
        return f"{self.user.user_name}"


# 3、User_Detail_Info 详细信息表 [1-1]
class DetailInfo(models.Model):
    user = models.ForeignKey(BaseInfo, verbose_name='用户', unique=True, on_delete=models.DO_NOTHING, )
    real_name = models.CharField(verbose_name='真实姓名', max_length=255, blank=True, null=True, db_index=True, )
    sex = models.CharField(verbose_name='性别', max_length=1, blank=True, null=True, )
    birth = models.DateField(verbose_name='生日', blank=True, null=True, )
    tags = models.CharField(verbose_name='个人标签', max_length=255, blank=True, null=True, )
    signature = models.CharField(verbose_name='个性签名', max_length=255, blank=True, null=True, )
    avatar = models.CharField(verbose_name='个人头像', max_length=255, blank=True, null=True, )
    cover = models.CharField(verbose_name='个人封面', max_length=255, blank=True, null=True, )
    language = models.CharField(verbose_name='语言', max_length=32, blank=True, null=True, )
    region_code = models.BigIntegerField(verbose_name='地区编码', blank=True, null=True, db_index=True, )
    more = models.JSONField(verbose_name='更多信息', max_length=255, blank=True, null=True, )
    field_1 = models.CharField(verbose_name='字段1', max_length=255, blank=True, null=True, )
    field_2 = models.CharField(verbose_name='字段2', max_length=255, blank=True, null=True, )
    field_3 = models.CharField(verbose_name='字段3', max_length=255, blank=True, null=True, )
    field_4 = models.CharField(verbose_name='字段4', max_length=255, blank=True, null=True, )
    field_5 = models.CharField(verbose_name='字段5', max_length=255, blank=True, null=True, )
    field_6 = models.CharField(verbose_name='字段6', max_length=255, blank=True, null=True, )
    field_7 = models.CharField(verbose_name='字段7', max_length=255, blank=True, null=True, )
    field_8 = models.CharField(verbose_name='字段8', max_length=255, blank=True, null=True, )
    field_9 = models.CharField(verbose_name='字段9', max_length=255, blank=True, null=True, )
    field_10 = models.CharField(verbose_name='字段10', max_length=255, blank=True, null=True, )
    field_11 = models.CharField(verbose_name='字段11', max_length=255, blank=True, null=True, )
    field_12 = models.CharField(verbose_name='字段12', max_length=255, blank=True, null=True, )
    field_13 = models.CharField(verbose_name='字段13', max_length=255, blank=True, null=True, )
    field_14 = models.CharField(verbose_name='字段14', max_length=255, blank=True, null=True, )
    field_15 = models.CharField(verbose_name='字段15', max_length=255, blank=True, null=True, )

    class Meta:
        db_table = 'user_detail_info'
        verbose_name_plural = "03. 用户 - 详细信息"

    def __str__(self):
        # return f"{self.user_name}({self.real_name})"
        return f"{self.user.user_name}"


class ExtendFiled(models.Model):
    field = models.CharField(verbose_name='自定义字段', max_length=255, unique=True, blank=True, null=True, db_index=True, )
    field_index = models.CharField(verbose_name='映射索引名', max_length=255, unique=True, blank=True, null=True, )
    description = models.CharField(verbose_name='字段描述', max_length=255, blank=True, null=True, )

    class Meta:
        db_table = 'user_extend_field'
        verbose_name_plural = "04. 用户 - 扩展字段表"

    def __str__(self):
        return f"{self.field}"


# 4、*User_Access_Log 访问日志表 [1-N]
class AccessLog(models.Model):
    user = models.ForeignKey(BaseInfo, verbose_name='用户', on_delete=models.DO_NOTHING, )
    ip = models.CharField(verbose_name='IP', max_length=128, blank=True, null=True, )
    create_time = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, )
    client_info = models.JSONField(verbose_name='客户端信息', blank=True, null=True, )
    more = models.JSONField(verbose_name='更多信息', max_length=255, blank=True, null=True, )

    class Meta:
        db_table = 'user_access_log'
        verbose_name_plural = "04. 用户 - 访问日志表"

    def __str__(self):
        # return f"{self.user_name}({self.full_name})"
        return f"{self.user.user_name}"


# 5、*User_History 操作历史表 [1-N]
class History(models.Model):
    user = models.ForeignKey(BaseInfo, verbose_name='用户', on_delete=models.DO_NOTHING, )
    field = models.CharField(verbose_name='操作字段', max_length=32, blank=True, null=True, )
    old_value = models.CharField(verbose_name='旧值', max_length=255, blank=True, null=True, )
    new_value = models.CharField(verbose_name='新值', max_length=255, blank=True, null=True, )
    create_time = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, )

    class Meta:
        db_table = 'user_history'
        verbose_name_plural = "05. 用户 - 操作历史表"

    def __str__(self):
        # return f"{self.user_name}({self.full_name})"
        return f"{self.user.user_name}"


# 6、*User_Restrict_Region 限制范围表
class RestrictRegion(models.Model):
    user = models.ForeignKey(BaseInfo, verbose_name='用户', on_delete=models.DO_NOTHING, )
    region_code = models.BigIntegerField(verbose_name='允许的地区编码', blank=True, null=True, db_index=True, )

    class Meta:
        db_table = 'user_restrict_region'
        verbose_name_plural = "06. 用户 - 限制范围表"

    def __str__(self):
        # return f"{self.user_name}({self.full_name})"
        return f"{self.user.user_name}"


# 7、*User_Platform 平台表
class Platform(models.Model):
    platform_id = models.IntegerField(verbose_name='平台ID', primary_key=True, help_text='必填。不自动生成，由运营人员统一设置。')
    platform_name = models.CharField(verbose_name='平台名称', max_length=128, help_text='必填。平台名称可以是中文、英文、俄文等。')
    platforms_to_users = models.ManyToManyField(BaseInfo, through='PlatformsToUsers', )

    class Meta:
        db_table = 'user_platform'
        verbose_name_plural = "07. 用户 - 平台表"

    def __str__(self):
        return f"{self.platform_name}"

    # 以上代码执行的Sql语句
    # CREATE TABLE `user_platform`(`platform_id` integer NOT NULL PRIMARY KEY, `platform_name` varchar(128) NOT NULL);


# 8、*User_Platforms_To_Users - 多对多平台记录表 [N-N]
class PlatformsToUsers(models.Model):
    user = models.ForeignKey(BaseInfo, verbose_name='用户', on_delete=models.DO_NOTHING, db_column='user_id')
    platform = models.ForeignKey(Platform, verbose_name='平台', on_delete=models.DO_NOTHING, )
    platform_user_id = models.BigIntegerField(verbose_name='平台用户ID', blank=True, null=True, db_index=True, )

    class Meta:
        db_table = 'user_platforms_to_users'
        verbose_name_plural = "08. 用户 - 多对多平台记录表"
        managed = False

    def __str__(self):
        # return f"{self.user_name}({self.full_name})"
        return f"{self.user.user_name}"

# 12、*user_contact_book - 一对多联系簿表 [1-N]
class ContactBook(models.Model):
    # user = models.ForeignKey(BaseInfo, verbose_name='用户', on_delete=models.DO_NOTHING, )
    user_id = models.IntegerField(verbose_name='用户')
    friend = models.ForeignKey(BaseInfo, verbose_name='朋友', on_delete=models.DO_NOTHING, )
    remarks = models.CharField(verbose_name='描述', max_length=32, blank=True, null=True, )

    class Meta:
        db_table = 'user_contact_book'
        verbose_name_plural = "09. 用户 - 联系簿"

    def __str__(self):
        # return f"{self.user_name}({self.full_name})"
        return f"{self.user_name}"



# Generated by Django 3.2.5 on 2022-02-05 05:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('xj_user', '0005_auth'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='auth',
            options={'verbose_name_plural': '用户认证'},
        ),
        migrations.AlterModelOptions(
            name='baseinfo',
            options={'verbose_name_plural': '账户 - 基础信息'},
        ),
        migrations.AlterModelOptions(
            name='platform',
            options={'verbose_name_plural': '交易平台'},
        ),
    ]



# Generated by Django 3.2.5 on 2022-01-01 09:19

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('xj_user', '0004_auto_20220101_1450'),
        ('xj_user', '0004_auto_20220101_1707'),
    ]

    operations = [
    ]



# encoding: utf-8
"""
@project: djangoModel->Auth
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 小程序SDK
@created_time: 2022/7/7 9:38
"""
from datetime import datetime, timedelta

from django.contrib.auth.hashers import make_password
import jwt
import redis
import requests

from config.config import Config
from ..models import BaseInfo, Auth
from ..utils.model_handle import parse_model


class WechatService:
    wx_login_url = "https://api.weixin.qq.com/sns/jscode2session"
    wx_token_url = 'https://api.weixin.qq.com/cgi-bin/token'
    wx_get_phone_url = "https://api.weixin.qq.com/wxa/business/getuserphonenumber"

    def __init__(self):
        self.login_param = {'appid': Config.getIns().get("xj_user", 'app_id'), 'secret': Config.getIns().get("xj_user", 'secret'), 'grant_type': 'authorization_code'}
        self.redis = redis.Redis(
            host=Config.getIns().get("main", 'redis_host'),
            port=Config.getIns().get("main", 'redis_port'),
            password=Config.getIns().get("main", 'redis_password')
        )

    def get_openid(self, code):
        """
        :param code（openid登录的code）:
        :return:(err,data)
        """
        try:
            response = requests.get(self.wx_login_url, code).json()
            if not response['errcode'] == 0:  # openid 换取失败
                return response['errcode'], response['errmsg']
        except Exception as e:
            return 6445, '请求错误'

    def phone_login(self, code):
        """
        过期续约就是重新登录
        :param code: 换取手机号码的code
        :return:(err,data)
        """
        # code 换取手机号
        url = self.wx_get_phone_url + "?access_token={}".format(self.get_access_token()['access_token'])
        header = {'content-type': 'application/json'}
        response = requests.post(url, json={'code': code}, headers=header).json()
        if not response['errcode'] == 0:
            return response['errmsg'], ""
        phone = response['phone_info']['phoneNumber']
        # 检查是否存在该用户，不存在直接注册
        current_user = BaseInfo.objects.filter(phone=phone).filter()
        current_user = parse_model(current_user)
        if not current_user:
            base_info = {
                'user_name': '',
                'phone': phone,
                'email': '',
                'full_name': '请修改用户名',
            }
            current_user = BaseInfo.objects.create(**base_info)
            current_user = parse_model(current_user)
        current_user = current_user[0]
        # 生成登录token
        token = self.__set_token(current_user.get('id', None), phone)
        # 创建用户登录信息，绑定token
        auth = {
            'user_id': current_user.get('id', None),
            'password': make_password('123456', None, 'pbkdf2_sha1'),
            'plaintext': '123456',
            'token': token,
        }
        Auth.objects.update_or_create({'user_id': current_user.get('id', None)}, **auth)
        return 0, {'token': token, 'user_info': current_user}

    def __set_token(self, user_id, account):
        # 生成过期时间
        expire_timestamp = datetime.utcnow() + timedelta(
            days=int(Config.getIns().get('xj_user', 'EXPIRE_DAY')),
            seconds=int(Config.getIns().get('xj_user', 'EXPIRE_SECOND'))
        )
        # 返回token
        return jwt.encode(
            payload={'user_id': user_id, 'account': account, "exp": expire_timestamp},
            key=Config.getIns().get('xj_user', 'JWT_SECRET_KEY')
        )

    def get_access_token(self):
        access_token = self.redis.get('access_token')
        if access_token:
            ttl = self.redis.ttl('access_token')
            return {"access_token": access_token.decode('utf-8'), 'expires_in': ttl, 'local': True}
        param = {
            'appid': Config.getIns().get("xj_user", 'app_id'),
            'secret': Config.getIns().get("xj_user", 'secret'),
            'grant_type': 'client_credential'
        }
        response = requests.get(self.wx_token_url, param).json()
        if 'access_token' in response.keys():
            self.redis.set('access_token', response['access_token'])
            self.redis.expire('access_token', response['expires_in'])
        return response

    def __del__(self):
        self.redis.close()



# Generated by Django 3.2.5 on 2022-01-01 06:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('xj_user', '0003_auto_20210511_1250'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='baseinfo',
            options={'verbose_name_plural': '账户基础信息表'},
        ),
        migrations.AlterModelOptions(
            name='platform',
            options={'verbose_name_plural': '平台'},
        ),
    ]



# encoding: utf-8
"""
@project: djangoModel->custom_merge
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 用户自定义 字典合并
@created_time: 2022/8/16 15:40
"""


class JoinList:
    def __init__(self, l_list, r_list, l_key="id", r_key="id"):
        self.l_key = l_key
        self.r_key = r_key
        self.l_list = l_list
        self.r_list = r_list
        self.l_k_list = [i[l_key] for i in l_list]
        self.r_k_list = [i[r_key] for i in r_list]

    def left_join(self):
        if not len(self.r_k_list) == len(self.r_list):
            raise Exception("r_key 不是唯一字段,无法映射")
        r_map = {item[self.r_key]: item for item in self.r_list}
        for item in self.l_list:
            if item[self.l_key] in r_map.keys():
                item.update(r_map[item[self.l_key]])
        return self.l_list

    def right_join(self):
        if not len(self.l_k_list) == len(self.l_list):
            raise Exception("r_key 不是唯一字段,无法映射")
        l_map = {item[self.l_key]: item for item in self.l_list}
        for item in self.r_list:
            if item[self.r_key] in l_map.keys():
                item.update(l_map[item[self.r_key]])
        return self.r_list



# _*_coding:utf-8_*_

# import os, logging, time, json, copy
import re

# from django.db.models import Q
# from django.db.models import F
from django.contrib.auth.hashers import make_password
from django.db.models import Q
import jwt
# from rest_framework.response import Response
from rest_framework import response
# from rest_framework import serializers
# from rest_framework import generics
# from rest_framework.permissions import IsAuthenticated
# from rest_framework.permissions import IsAdminUser
from rest_framework.permissions import AllowAny
from rest_framework.views import APIView

from config.config import Config
# import random
from xj_user.models import *


# from datetime import datetime, timedelta
# from django.contrib.auth.hashers import check_password
# from django.conf import settings

# logger = logging.getLogger(__name__)


# 用户注册
class UserRegister(APIView):
    permission_classes = (AllowAny,)
    model = BaseInfo
    params = None

    def post(self, request, *args, **kwargs):
        # self.params = request.query_params  # 返回QueryDict类型
        self.params = request.data  # 返回QueryDict类型
        # self.params.update(request.data)
        # print('> self.params:', self.params)
        token = None

        try:
            account = str(self.params.get('account', ''))
            password = str(self.params.get('password', ''))
            platform = str(self.params.get('platform', ''))
            full_name = str(self.params.get('full_name', ''))

            # 边界检查
            if not account:
                raise MyApiError("account必填", 2001)

            if not password:
                raise MyApiError("password必填", 2003)

            # if not platform:
            #     raise MyApiError("platform必填", 2007)

            if not full_name:
                raise MyApiError("full_name必填", 2008)

            # 检查平台是否存在
            # platform_set = Platform.objects.filter(platform_name__iexact=platform)
            # if platform_set.count() == 0:
            #     raise MyApiError("platform不存在平台名称："+platform, 2009)
            # platform_id = platform_set.first().platform_id

            # 账号类型判断
            if re.match(r'(^1[356789]\d{9}$)|(^\+?[78]\d{10}$)', account):
                account_type = 'phone'
            elif re.match(r'^\w+[\w\.\-\_]*@\w+[\.\w]*\.\w{2,}$', account):
                account_type = 'email'
            elif re.match(r'^[A-z\u4E00-\u9FA5]+\w*$', account):
                account_type = 'username'
            else:
                raise MyApiError("账号必须是用户名、手机或者邮箱，用户名不能是数字开头", 2009)
            # print('> account_type:', account_type)

            # 检查账号是否存在
            user_list = None
            if account_type == 'phone':
                user_list = BaseInfo.objects.filter(Q(phone=account))
            elif account_type == 'email':
                user_list = BaseInfo.objects.filter(Q(email=account))
            elif account_type == 'username':
                user_list = BaseInfo.objects.filter(Q(user_name=account))
            # print("> user_list:", user_list)

            if user_list.count() and account_type == 'phone':
                raise MyApiError("手机已被注册: " + account)
            elif user_list.count() and account_type == 'email':
                raise MyApiError("邮箱已被注册: " + account)
            elif user_list.count() and account_type == 'username':
                raise MyApiError("用户名已被注册: " + account)

            base_info = {
                # 'platform_uid': round(random.random()*1000000000),
                # 'platform_id': platform_id,
                'user_name': account if account_type == 'username' else '',
                'phone': account if account_type == 'phone' else '',
                'email': account if account_type == 'email' else '',
                'full_name': full_name,
            }
            # print("> base_info:", base_info)
            current_user = BaseInfo.objects.create(**base_info)
            # print("> current_user:", current_user.id)

            # SECURITY WARNING: keep the secret key used in production secret!
            SECRET_KEY = 'django-insecure-l1$-m!u=!f9&o2$f(cm#dasus&a=5i1#+kh)090_=p%+==%9o1'
            JWT_SECRET_KEY = '@xzm2021!'

            token = jwt.encode({'account': account}, Config.getIns().get('xj_user', "JWT_SECRET_KEY"))
            auth = {
                'user_id': current_user.id,
                'password': make_password(password, None, 'pbkdf2_sha1'),
                'plaintext': password,
                # 'token': jwt.encode(current_user.id, settings.JWT_SECRET_KEY),
                'token': token,
            }
            # print("> auth:", auth)
            # print("> check_password:", check_password(password, 'pbkdf2_sha1$260000$jNjzstMtWofWN0tmUzfQwh$02D9zbyiPyYHZwb2OdrpmTwN8zI='))

            current_auth = Auth.objects.create(**auth)

            res = {
                'err': 0,
                'msg': '注册成功',
                'data': {
                    "user_id": current_user.id,
                    # "token": token,
                },
            }

        except SyntaxError:
            # print("> SyntaxError:")
            res = {
                'err': 4001,
                'msg': '语法错误'
            }
        except LookupError:
            res = {
                'err': 4002,
                'msg': '无效数据查询'
            }
        # 这里 result是一个类的对象，要用result.属性名来返回
        except Exception as valueError:
            # print('> Exception:', valueError)
            res = {
                'err': valueError.err if hasattr(valueError, 'err') else 4000,
                'msg': valueError.msg if hasattr(valueError, 'msg') else valueError.args,
                # 'request': self.params,
            }
        except:
            res = {
                'err': 4999,
                'msg': '未知错误'
            }

        # return super(UserLogin, self).patch(request, *args, **kwargs)

        # return super(UserLogin, self).patch(request, *args, **kwargs)
        headers = {
            "Authorization": token,
        }
        return response.Response(data=res, status=None, template_name=None, headers=headers, content_type=None)


class MyApiError(Exception):
    def __init__(self, message, err_code=4010):
        self.msg = message
        self.err = err_code

    def __str__(self):
        # repr()将对象转化为供解释器读取的形式。可省略
        return repr(self.msg)
